package com.diagbot.facade;

import com.diagbot.client.CRFServiceClient;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.model.ai.AIAnalyze;
import com.diagbot.model.label.ChiefLabel;
import com.diagbot.model.label.DiagLabel;
import com.diagbot.model.label.PastLabel;
import com.diagbot.model.label.PresentLabel;
import com.diagbot.util.CoreUtil;
import com.diagbot.vo.SearchData;
import com.diagbot.vo.StandConvert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Description: 通过业务facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class CommonFacade {
    @Autowired
    CRFServiceClient crfServiceClient;

    //组装好的label
    public WordCrfDTO crf_process(SearchData searchData){
        AIAnalyze aiAnalyze = new AIAnalyze(crfServiceClient);
        WordCrfDTO wordCrfDTO = new WordCrfDTO();
        aiAnalyze.aiProcess(searchData, wordCrfDTO);
        return wordCrfDTO;
    }

    //返回给图谱的词，这些词需要转换成标准词
    public StandConvert dataTypeGet(WordCrfDTO wordCrfDTO){
        StandConvert standConvert = new StandConvert();

        //所有的症状(主诉、现病史)
        List<String> clinicalList = new ArrayList<>();
        ChiefLabel chiefLabel = wordCrfDTO.getChiefLabel();
        clinicalList.addAll(CoreUtil.getPropertyList(chiefLabel.getClinicals()));
        PresentLabel presentLabel = wordCrfDTO.getPresentLabel();
        clinicalList.addAll(CoreUtil.getPropertyList(presentLabel.getClinicals()));

        //所有的疾病(主诉、现病史,诊断列表,过去史)
        DiagLabel diagLabel = wordCrfDTO.getDiagLabel();
        PastLabel pastLabel = wordCrfDTO.getPastLabel();
        List<String> diagList = new ArrayList<>();
        diagList.addAll(CoreUtil.getPropertyList(chiefLabel.getDiags()));
        diagList.addAll(CoreUtil.getPropertyList(presentLabel.getDiags()));
        diagList.addAll(CoreUtil.getPropertyList(diagLabel.getDiags()));
        diagList.addAll(CoreUtil.getPropertyList(pastLabel.getDiags()));

        standConvert.setClinicalList(clinicalList);
        standConvert.setDiaglList(diagList);
        return standConvert;
    }
    //把图谱返回的标准词set到label中
    public WordCrfDTO dataTypeSet(WordCrfDTO wordCrfDTO, Map<String, Map<String, String>> map){
        ChiefLabel chiefLabel = wordCrfDTO.getChiefLabel();
        CoreUtil.setPropertyList(chiefLabel.getClinicals(),map.get("clinicalList"));
        return wordCrfDTO;
    }

}
